#include "dl_ws_client.h"

DL_WS_CLIENT::DL_WS_CLIENT()
{
  ServerSocket = INVALID_SOCKET;
  
  memset (nick,'\0',32);
  memset (passwd,'\0',32);
  memset (serverip,'\0',16);
  memset (serverport,'\0',8);
  memset (sendbuffer,'\0',128);
  memset (recvbuffer,'\0',128);
  ID = -1;
}

DL_WS_CLIENT::~DL_WS_CLIENT()
{
  
  // Set blocking socket
  u_long iMode = 0;
  ioctlsocket(ServerSocket, FIONBIO, &iMode);
  disconnect();
  deinit_ws();
}

int DL_WS_CLIENT::init_ws()
{
  clientlog = fopen("clientlog.txt","w");

  // Initialize Winsock
  int iResult = WSAStartup(MAKEWORD(2,2), &wsaData);
  if (iResult != 0) {
      printf("WSAStartup failed: %d\n", iResult);
      fclose(clientlog);
      WSACleanup (); // unload ws2_32.dll
      return 0;
  }
  /* ok, we started our WSA successfully */ 
  fclose(clientlog);
  return 1;
}

int DL_WS_CLIENT::deinit_ws()
{
  WSACleanup();
  return 1;
}

int DL_WS_CLIENT::connect_to_server()
{
  clientlog = fopen("clientlog.txt","w");
  
  struct addrinfo *result = NULL,
                  *ptr = NULL,
                  hints;
  
  ZeroMemory( &hints, sizeof(hints) );
  hints.ai_family   = AF_UNSPEC;
  hints.ai_socktype = SOCK_STREAM;
  hints.ai_protocol = IPPROTO_TCP;

  // Resolve the server address and port
  int iResult = getaddrinfo(serverip, serverport, &hints, &result);
  if ( iResult != 0 ) {
      printf("getaddrinfo failed: %d\n", iResult);
      fclose(clientlog);
      WSACleanup ();
      return 0;
  }

  // Attempt to connect to an address until one succeeds
  for( ptr = result; ptr != NULL ; ptr = ptr->ai_next )
  {
    // Create a SOCKET for connecting to server
    ServerSocket = socket(ptr->ai_family, ptr->ai_socktype, 
        ptr->ai_protocol);
    if (ServerSocket == INVALID_SOCKET) {
        printf("Error at socket(): %ld\n", WSAGetLastError());
        freeaddrinfo(result);
        fclose(clientlog);
        WSACleanup();
        return 0;
    }
    
    // Connect to server.
    iResult = connect( ServerSocket, ptr->ai_addr, (int)ptr->ai_addrlen);
    if (iResult == SOCKET_ERROR) {
        closesocket(ServerSocket);
        ServerSocket = INVALID_SOCKET;
        continue;
    }
    
    break;
  }

  freeaddrinfo(result);
  
  // Set non-blocking socket
  u_long iMode = 1;
  ioctlsocket(ServerSocket, FIONBIO, &iMode);
  
  if (ServerSocket == INVALID_SOCKET) {
      printf("Unable to connect to server!\n");
      fclose(clientlog);
      WSACleanup();
      return 0;
  }

  fclose(clientlog);
  return 1;
}

int DL_WS_CLIENT::disconnect()
{
  if(ServerSocket != INVALID_SOCKET)
  {
    dl_send("03");
    while(1)
    {
      //printf("still receiving\n");
      dl_recv();
      if(recvbuffer[0] == '5' && recvbuffer[1] == '0') break;
    }
    closesocket(ServerSocket);
    ServerSocket = INVALID_SOCKET;
  }
}

int DL_WS_CLIENT::set_account(const char *nick, const char *passwd)
{
  strcpy(this->nick, nick);
  strcpy(this->passwd, passwd);
}

int DL_WS_CLIENT::select_server(const char *ip, const char *port)
{
  strcpy(serverip, ip);
  strcpy(serverport, port);
}

int DL_WS_CLIENT::dl_send(const char* buffer)
{
  memset(sendbuffer, '\0', 128);
  sprintf (sendbuffer, buffer);
  //printf("buffer: %s\n", sendbuffer);
  send (ServerSocket, sendbuffer, sizeof(sendbuffer), 0);
}

char* DL_WS_CLIENT::dl_recv()
{
  memset(recvbuffer, '\0', 128);
  int res = recv (ServerSocket, recvbuffer, sizeof(recvbuffer), 0);
  //if(res > 0)
    //printf("%s\n", recvbuffer);
  return recvbuffer;
}
