#include "dl_hge_paper.h"

PAPER::PAPER(HGE *h)
{
  //printf("spravil som novy paper\n");
  m_hge = h;
}

PAPER::~PAPER()
{
  //printf("znicil som papier\n");
}

int PAPER::draw(int *ap)
{
  if(*ap == ID)
  {
    m_hge->Gfx_RenderQuad(&bg);
    
    list<dlBITMAP>::iterator itbmp;
    for(itbmp = bitmaps.begin(); itbmp != bitmaps.end(); itbmp++)
    {
      itbmp->draw();
    }
    
    list<BUTTON>::iterator itbut;
    for(itbut = buttons.begin(); itbut != buttons.end() ; itbut++)
    {
      itbut->draw();
    }
    
    list<PARTICLE>::iterator itpar;
    for(itpar = particles.begin(); itpar != particles.end(); itpar++)
    {
      itpar->draw();
    }
    
    list<CHECKBOX>::iterator itcbx;
    for(itcbx = checkboxes.begin(); itcbx != checkboxes.end(); itcbx++)
    {
      itcbx->draw();
    }
    
    list<RADIOGROUP>::iterator itrgp;
    for(itrgp = radiogroups.begin(); itrgp != radiogroups.end(); itrgp++)
    {
      itrgp->draw();
    }
    
    list<SLIDER>::iterator itsld;
    for(itsld = sliders.begin(); itsld != sliders.end(); itsld++)
    {
      itsld->draw();
    }
    
    list<LISTBOX>::iterator itlbx;
    for(itlbx = listboxes.begin(); itlbx != listboxes.end(); itlbx++)
    {
      itlbx->draw();
    }
    
    list<TEXTBOX>::iterator ittbx;
    for(ittbx = textboxes.begin(); ittbx != textboxes.end(); ittbx++)
    {
      ittbx->draw();
    }
    
    //printf("paper %d drawn\n", ID);
  }
  return 1;
}

int PAPER::manage(int *ap)
{
  float dt = m_hge->Timer_GetDelta();
  
  if(*ap == ID)
  {
    list<BUTTON>::iterator itbut;
    for(itbut = buttons.begin(); itbut != buttons.end() ; itbut++)
    {
      if(itbut->manage(ap, dt) == 0) return 0;
    } 
    
    list<PARTICLE>::iterator itpar;
    for(itpar = particles.begin(); itpar != particles.end(); itpar++)
    {
      itpar->manage(dt);
    }
    
    list<CHECKBOX>::iterator itcbx;
    for(itcbx = checkboxes.begin(); itcbx != checkboxes.end(); itcbx++)
    {
      itcbx->manage();
    }
    
    list<RADIOGROUP>::iterator itrgp;
    for(itrgp = radiogroups.begin(); itrgp != radiogroups.end(); itrgp++)
    {
      itrgp->manage();
    }
    
    list<SLIDER>::iterator itsld;
    for(itsld = sliders.begin(); itsld != sliders.end(); itsld++)
    {
      itsld->manage();
    }
    
    list<LISTBOX>::iterator itlbx;
    for(itlbx = listboxes.begin(); itlbx != listboxes.end(); itlbx++)
    {
      itlbx->manage(ap, dt);
    }
    
    list<TEXTBOX>::iterator ittbx, ittbxs;
    for(ittbx = textboxes.begin(); ittbx != textboxes.end(); ittbx++)
    {
      if(mir(m_hge, ittbx->x, ittbx->y, ittbx->x+ittbx->w, ittbx->y+ittbx->h, 1))
      {
        ittbx->active = 1;
        for(ittbxs = textboxes.begin(); ittbxs != textboxes.end(); ittbxs++)
        {
          if(ittbxs != ittbx)ittbxs->active = 0;
        }
      }
      if(ittbx->active)
        ittbx->manage();
    }
    
    //printf("paper %d managed\n", ID);
  }
  return 1;
}

int PAPER::set_background(char *filename, DWORD col)
{
  //printf("col:%d %d\n", col, 0xffffffff);
  bg = *dl_util_makeHgeQuad(m_hge, 0, 0, m_hge->System_GetState(HGE_SCREENWIDTH), m_hge->System_GetState(HGE_SCREENHEIGHT), col, filename);
  //printf("set\n");
  return 1;
}

int PAPER::add_button(int x, int y, int w, int h, int id, int type, int way, void(*proc)())
{
  BUTTON b(m_hge);
  
  b.x = x;
  b.y = y;
  b.w = w;
  b.h = h;
  b.ID = id;
  b.type = type;
  b.way = way;
  b.proc = proc;
  
  buttons.push_back(b);
  //printf("button pushed back %d", b.ID);
  
  return 1;
}

int PAPER::add_particle(char *particle_system, char *particle_texture, int tex_x, int tex_y, int tex_w, int tex_h, int x, int y, int id, int mode)
{
  PARTICLE p(m_hge);
  
  p.x = x;
  p.y = y;
  p.ID = id;
  p.mode = mode;
  HTEXTURE tex = m_hge->Texture_Load(particle_texture);
  p.spr = new hgeSprite(tex, tex_x, tex_y, tex_w, tex_h);
  p.spr->SetBlendMode(BLEND_COLORMUL | BLEND_ALPHAADD | BLEND_NOZWRITE);
  p.spr->SetHotSpot(tex_w/2, tex_h/2);
  p.par = new hgeParticleSystem(particle_system, p.spr);
  p.par->MoveTo(x,y);
  if(mode == PAR_FIRE_ON_START)
    p.par->Fire();
  
  particles.push_back(p);
  return 1;
}

int PAPER::add_checkbox(int x, int y, int w, int h, int id, void(*proc)(bool))
{
  CHECKBOX c(m_hge);
  
  c.x = x;
  c.y = y;
  c.w = w;
  c.h = h;
  c.ID = id;
  c.proc = proc;
  
  checkboxes.push_back(c);
  return 1;
}

int PAPER::add_radiogroup(int x, int y, int klesanie, int id, void(*proc)(int))
{
  RADIOGROUP r(m_hge);
  
  r.x = x;
  r.y = y;
  r.klesanie = klesanie;
  
  r.ID = id;
  r.proc = proc;
  
  radiogroups.push_back(r);
  return 1;
}

int PAPER::add_slider(int x, int y, int w, int h, int id, void(*proc)(int))
{
  SLIDER s(m_hge);
  
  s.x = x;
  s.y = y;
  s.w = w;
  s.h = h;
  
  s.ID = id;
  s.proc = proc;
  
  sliders.push_back(s);
  return 1;
}
//name bug, nie vlak_w ale kolajnica_w...
int PAPER::add_listbox(int x, int y, int w, int h, int id, int vlak_w, void(*proc)(int, char*))
{
  LISTBOX l(m_hge);
  
  l.x = x;
  l.y = y;
  l.w = w;
  l.h = h;
  l.kolajnica_w = vlak_w;
  
  l.ID = id;
  l.proc = proc;
  
  listboxes.push_back(l);
  return 1;
}

int PAPER::add_bitmap(int id)
{
  dlBITMAP b(m_hge);
  
  b.ID = id;
  bitmaps.push_back(b);
  return 1;
}

int PAPER::add_textbox(int x, int y, int w, int h, int id)
{
  TEXTBOX t(m_hge);
  
  t.ID = id;
  t.x = x;
  t.y = y;
  t.w = w;
  t.h = h;
  
  textboxes.push_back(t);
  return 1;
}

list<BUTTON>::iterator PAPER::search_button(int id)
{
  list<BUTTON>::iterator it;
  
  for(it = buttons.begin(); it!= buttons.end(); it++)
  {
    if(it->ID == id) break;
  }
  //printf("nasiel som button %d\n", it->ID);
  return it;
}

list<PARTICLE>::iterator PAPER::search_particle(int id)
{
  list<PARTICLE>::iterator it;
  
  for(it = particles.begin(); it != particles.end(); it++)
  {
    if(it->ID == id) break;
  }
  //printf("nasiel som particle %d\n", it->ID);
  return it;
}

list<CHECKBOX>::iterator PAPER::search_checkbox(int id)
{
  list<CHECKBOX>::iterator it;
  
  for(it = checkboxes.begin(); it != checkboxes.end(); it++)
  {
    if(it->ID == id) break;
  }
  
  return it;
}

list<RADIOGROUP>::iterator PAPER::search_radiogroup(int id)
{
  list<RADIOGROUP>::iterator it;
  for(it = radiogroups.begin(); it != radiogroups.end(); it++)
  {
    if(it->ID == id) break;
  }
  return it;
}

list<SLIDER>::iterator PAPER::search_slider(int id)
{
  list<SLIDER>::iterator it;
  for(it = sliders.begin(); it != sliders.end(); it++)
  {
    if(it->ID == id) break;
  }
  return it;
}

list<dlBITMAP>::iterator PAPER::search_bitmap(int id)
{
  list<dlBITMAP>::iterator it;
  for(it = bitmaps.begin(); it != bitmaps.end(); it++)
  {
    if(it->ID == id) break;
  }
  return it;
}

list<LISTBOX>::iterator PAPER::search_listbox(int id)
{
  list<LISTBOX>::iterator it;
  for(it = listboxes.begin(); it != listboxes.end(); it++)
  {
    if(it->ID == id) break;
  }
  return it;
}

list<TEXTBOX>::iterator PAPER::search_textbox(int id)
{
  list<TEXTBOX>::iterator it;
  for(it = textboxes.begin(); it != textboxes.end(); it++)
  {
    if(it->ID == id) break;
  }
  return it;
}
